function [] = main(dataset,experiment,run)
format compact;
% in case we use the matlab compiler
if isdeployed
   dataset=str2num(dataset);
   experiment=str2num(experiment);
   run=str2num(run);
end

tabdatasets={'covtype','alpha','ocr','real-sim','rcv1','webspam'};
path_datasets='/scratch2/clear/mairal/large_datasets/';
path_logs='/scratch2/clear/mairal/large_datasets/logs_siam_revision/';
tabepochs=[1 2 3 4 5 10 15 20 25 50];
addpath('SAG');  % this is the path to SAG, SDCA, ...
addpath('../../trunk/build/');  % this is the path to SPAMS
% compilation options for SPAMS should be compiler='gcc'; blas='builtin';  such
% that all methods are linked to the same BLAS library
% Matlab should be run with the option -singleCompThread  

rand('seed',run);
mkdir(path_logs);

% value for experiment can be
% 1: SGD-h
% 2: FISTA
% 3: SDCA 
% 4: SAG
% 5: MISO0
% 6: MISO1
% 7: MISO2
% 8: MISOmu

%% Load datasets
tabn=[581012 500000 2500000 72309 781265 250000];
n=tabn(dataset);
dataset=tabdatasets{dataset};
tablambda=[1/n 10/n 0.1/n];
if (strcmp(dataset,'covtype') | strcmp(dataset,'alpha') | strcmp(dataset,'ocr'))
   param.normalized=false; 
   load([path_datasets dataset '.full_norm.mat']);
else
   param.normalized=true;   % we have normalized every dataset
   load([path_datasets dataset '.full.mat']);
end
C=unique(y);
neg=find(y==C(1));
y(neg)=-1;
pos=find(y==C(2));
y(pos)=1;
n=size(X,2);
p=size(X,1);
ind=randperm(n);
X=X(:,ind);
y=y(ind);
clear ind;

%%% Set Up Experiment
param.verbose=true;
param.numThreads=1;
param.regul='l2';
param.loss='logistic';

for ii = 1:length(tablambda)
   param.lambda=tablambda(ii);
   lambda=param.lambda;
   name=sprintf('%s/exp_%s_%s_%s_%d_%d_%d.mat',path_logs,'logistic','l2',dataset,ii,experiment,run);
   if exist(name) 
      continue;
   end

   ws=cell(1,length(tabepochs));
   optims=zeros(1,length(tabepochs));
   times=zeros(1,length(tabepochs));
   for kk = 1:length(tabepochs)
      epochs=tabepochs(kk);

      if experiment == 1 % SGD-h
         param.random=false;
         param.minibatches=1;
         param.optimized_solver=true;
         param.iters=ceil(epochs*n/param.minibatches);
         param.determineEta=true;
         param.averaging_mode=0;
         param.weighting_mode=0;  %  eta/(t+t0), 
         w0=zeros(p,1);
         [w wav optim] = mexStochasticProx(y,X,w0,param);
         t=optim(3);
         optim=optim(1);
      elseif experiment==2  % FISTA
         param.ista=false;
         param.max_it=tabepochs(kk);
         param.it0=5;
         param.verbose=true;
         param.tol=1e-10;
         w0=zeros(p,1);
         param.L0=mean(sum(X.^2,2)/size(X,2))/10;
         Xt=X';
         tic;
         [w optim] = mexFistaFlat(y,Xt,w0,param);
         t=toc;
         clear Xt;
      elseif experiment==3  % SDCA
         maxIter=epochs*n;
         iVals = int32(ceil(n*rand(maxIter,1)));
         w = zeros(p,1);
         yXt = X*diag(sparse(y));
         alpha = 1e-10*ones(n,1);
         tic
         if issparse(X)
            DCA_logistic(alpha,yXt,lambda,iVals);
         else
            yXt=full(yXt);
            DCA_logistic_BLAS(alpha,yXt,lambda,iVals);
         end
         w = yXt*alpha/(lambda*n);
         t=toc;
         clear yXt;
      elseif experiment==4  % SAG 1/L
         maxIter=epochs*n;
         iVals = int32(ceil(n*rand(maxIter,1)));
         w = zeros(p,1);
         L=max((.25)*sum(X.^2)) + lambda;
         stepSize = 1/L;   
         d = zeros(p,1);
         g = zeros(n,1);
         covered = int32(zeros(n,1));
         tic
         if issparse(X)
            SAG_logistic(w,X,y,lambda,stepSize,iVals,d,g,covered);
         else
            SAG_logistic_BLAS(w,X,y,lambda,stepSize,iVals,d,g,covered);
         end
         t=toc;
      elseif experiment <= 7  % the MISO experiments with minibatch for sparse matrices
         density=nnz(X)/(n*p);  
         param.minibatches=floor(1/density);    % will require storing twice the design matrix
         param.epochs=epochs;
         if (experiment == 5)
            param.strategy=0;      % MISO0, not very good, for comparison purposes
         elseif (experiment==6)
            param.strategy=1;      % MISO1, not as good as MISO2, for comparison purposes
         elseif (experiment==7)
            param.strategy=3;      % MISO2
         end
         w0=zeros(p,1);
         [w optim] = mexIncrementalProx(y,X,w0,param);
         t=optim(3);
         optim=optim(1);
      elseif experiment==8
         param.minibatches=1; 
         param.epochs=epochs;
         param.strategy=4; % MISOmu
         w0=zeros(p,1);
         [w optim] = mexIncrementalProx(y,X,w0,param);
         optim
         t=optim(3);
         optim=optim(1);
      end

      % evaluation of the cost function
      paramdummy=param;
      paramdummy.epochs=0;
      paramdummy.verbose=false;
      paramdummy.minibatches=size(X,2);
      paramdummy.strategy=0;
      [tmp optim] = mexIncrementalProx(y,X,w,paramdummy);
      ws{kk}=w;
      optims(kk)=optim(1);
      times(kk)=t;
      optims
      times
   end
   save(name,'ws','optims','tabepochs','times','-v7.3');
end
